package com.simualteSmack;

import java.io.IOException;
import java.io.Reader;
import java.util.ArrayList;
import java.util.List;
/**      
 * @date 2011-7-31  
 * @author zhanghaitao     
 * @version 1.0 
 *     
 */
public class ObservableReader extends Reader {

	Reader wrappedReader = null;
	List listeners = new ArrayList();

	public ObservableReader(Reader wrappedReader) {
		this.wrappedReader = wrappedReader;
	}

	@Override
	public int read(char[] cbuf, int off, int len) throws IOException {
		int count = wrappedReader.read(cbuf, off, len);
		if (count > 0) {
			String str = new String(cbuf, off, count);
			// Notify that a new string has been read
			ReaderListener[] readerListeners = null;
			synchronized (listeners) {
				readerListeners = new ReaderListener[listeners.size()];
				listeners.toArray(readerListeners);
			}
			for (int i = 0; i < readerListeners.length; i++) {
				readerListeners[i].read(str);
			}
		}
		return count;
	}

	@Override
	public void close() throws IOException {
		wrappedReader.close();
	}

	public int read() throws IOException {
		return wrappedReader.read();
	}

	public int read(char cbuf[]) throws IOException {
		return wrappedReader.read(cbuf);
	}

	public long skip(long n) throws IOException {
		return wrappedReader.skip(n);
	}

	public boolean ready() throws IOException {
		return wrappedReader.ready();
	}

	public boolean markSupported() {
		return wrappedReader.markSupported();
	}

	public void mark(int readAheadLimit) throws IOException {
		wrappedReader.mark(readAheadLimit);
	}

	public void reset() throws IOException {
		wrappedReader.reset();
	}

	/**
	 * Adds a reader listener to this reader that will be notified when new
	 * strings are read.
	 * 
	 * @param readerListener
	 *            a reader listener.
	 */
	public void addReaderListener(ReaderListener readerListener) {
		if (readerListener == null) {
			return;
		}
		synchronized (listeners) {
			if (!listeners.contains(readerListener)) {
				listeners.add(readerListener);
			}
		}
	}

	/**
	 * Removes a reader listener from this reader.
	 * 
	 * @param readerListener
	 *            a reader listener.
	 */
	public void removeReaderListener(ReaderListener readerListener) {
		synchronized (listeners) {
			listeners.remove(readerListener);
		}
	}

}
